/**
*	File: mst_stepper.js
*	Version: 1.0
*	Author: Wellcom Worldwide
*
*	Description: 
*
*	This file contains the stepper functionality used for Mercer Style Guide
*	Allows for vertical and horizontal initializations.
*	
*/

;(function($) {


	/**
	*	Check dependencies
	*/
	if(typeof $ == 'undefined' && typeof jQuery == 'undefined') {
		console.log('jQuery is undefined.');
		return;
	}




	$.fn.__mstStepper = function( opts ) {

		// Declare variables that are used for the stepper funtionality
		var selectedObjects = this;


		// Public settings and variables 
		var settings = $.extend({
			direction: 'horizontal',
			mobileBreakpoint: 768,
			titleClass: '.__mst_stepper_content_title',
			hideContent: false,
		}, opts);

		// Keep the original settings for later use
		settings.originalDirection = settings.direction.toLowerCase();
		settings.originalHideContent = settings.hideContent;

		var pluginMethods = {
			__init: function( selector ) {

				// console.log('Method is called!');
				var el = $( selector );


				if(settings.originalDirection == 'horizontal' && $(window).width() >= settings.mobileBreakpoint) {
					


					/** ;;; DEBUGGING ;;; **/

					// console.log('Original Direction is horizontal.');
					// console.log('WINDOW WIDTH: ' + $(window).width());

					// Check to see if the current direction before re-mutation is `vertical`; this means that a 
					// reset is required
					pluginMethods.resetContainer( el );




					settings.direction = settings.originalDirection;
					settings.hideContent = settings.originalHideContent;
					el.removeClass('__mst_stepper_vertical')
						.addClass('__mst_stepper');


					pluginMethods.calculateStageWidth( el );


				} else {

					settings.direction = 'vertical';


					if( settings.originalDirection == 'horizontal' ) {
						settings.hideContent = true;
						pluginMethods.removeStageStyles( el );
						// Now that the original Direction has been deteremined hide all of the content!
						pluginMethods.hideAllInactive( el );

					}

				}

				// Reset back to normal! Trigger a click!
				pluginMethods.resetBarStyles( el );
				pluginMethods.triggerResponsiveHorizontal( el );

				// console.log(settings.direction);


				// Apply the outer wrapper class 
				if(settings.direction.toLowerCase() == 'horizontal') {

					// console.log('H1Z1');

					// Check to see if the current window width is @ Mobile Breakpoint
					if( $(window).width() >= settings.mobileBreakpoint ) {

						if(!el.hasClass('__mst_stepper')) {
							el.addClass('__mst_stepper');
						}

						settings.direction = 'horizontal';

					} else {

						// Revert to Vertical Stepper
						el.removeClass('__mst_stepper')
							.addClass('__mst_stepper_vertical');

						settings.direction = 'vertical';

					}


				} else if (settings.direction.toLowerCase() == 'vertical') {

					// console.log('VZ1Z');

					if(!el.hasClass('__mst_stepper_vertical')) {
						el.addClass('__mst_stepper_vertical');
					}


					if(el.hasClass('__mst_stepper')) {

						el.removeClass('__mst_stepper');

					}
					
					
					pluginMethods.calculateListHeight( el );


				}



			},
			loopInit: function( selectedObjects ) {

				selectedObjects.each(function(i, o) {

					// Initialize each of the selectors
					pluginMethods.__init( $(this) );

					var el = $(this);
					pluginMethods.bindOnClickEvent( el );

				});

			},
			calculateStageWidth: function( el ) {


				// Now that it has been dteremined the stepper is horizontal, calculate the width to be applied to the element
				var listItems = el.find( '.__mst_stepper_list_item' );

				var minWidth = 20; // Extra 10 buffer both sides

				listItems.each( function( i, o ) {

					var inEl = $(this);
					var inElWidth = inEl[0].getBoundingClientRect().width;


					minWidth += Math.ceil( inElWidth );


				});



				var inElWrapper = el.find( '.__mst_stepper_list' );

				if( el.hasClass('__mst_stepper') ) {
					inElWrapper.css('min-width', minWidth + 'px');
				} else {
					inElWrapper.css('min-width', '');
				}

			},
			listItemOnClick: function( ev ) {


				// console.log('List Item is firing');


				var selector = $( ev.data.selector );

				var el = $( ev.currentTarget );

				// console.log( ev );


				if( $( ev.target ).hasClass( pluginMethods.returnTitleClass() ) ) {

					// Now we know that this is the title class,
					// we need to change the variables to match.
					var titlePar = $( ev.target ).parents('.__mst_stepper_content');
					var titleID = titlePar.attr('id');

					var el = $('.__mst_stepper_list_item[data-target="#' + titleID + '"]');

					// console.log(el);

				}


				var tg = $( el.attr('data-target') );
				var par = el.parents('.__mst_stepper_list');
				var wrap = el.parents('.__mst_stepper');
				var prev = el.prevAll('.__mst_stepper_list_item');
				var next = el.nextAll('.__mst_stepper_list_item');
				var all = par.find('.__mst_stepper_list_item');
				var content = wrap.find('.__mst_stepper_content');
				var contentWrapper = wrap.find('.__mst_stepper_content_wrapper');



				contentWrapper.addClass('active');
				content.removeClass('active');
				all.removeClass('current complete');

				prev.addClass('complete');

				el.addClass('current');


				tg.addClass('active');




				// Check to see if the current initialization direction is 'vertical' or 'horizontal'
				if( settings.direction.toLowerCase() === 'vertical' ) {

					// Now also check to see if all content should be hidden
					if ( settings.hideContent ) {

						// Find all within the target item that isn't the title
						// and apply a hidden class
						// var children = itemTarget.children().addClass('active');
						prev.each(function(i, o) {

							var inEl = $(this);


							// Now loop through and disable all of these targets
							var inElTarget = inEl.attr('data-target');

							if(typeof inElTarget != "undefined" && inElTarget != null && inElTarget != "")  {

								// Houston we have a hit on the target
								inElTarget = $( inElTarget );


								inElTarget.children().not( settings.titleClass ).removeClass('active').addClass('hidden');


							}


						});


						next.each(function(i, o) {

							var inEl = $(this);


							// Now loop through and disable all of these targets
							var inElTarget = inEl.attr('data-target');

							if(typeof inElTarget != "undefined" && inElTarget != null && inElTarget != "")  {

								// Houston we have a hit on the target
								inElTarget = $( inElTarget );


								inElTarget.children().not( settings.titleClass ).removeClass('active').addClass('hidden');


							}


						});



						// Now for the current target element.
						if(typeof tg != "undefined" && tg != null && tg != "") {

							var elTg = $( tg );

							tg.children().not( settings.titleClass ).removeClass('hidden').addClass('active');


						}



						pluginMethods.calculateListHeight( selector );


					}

				} else {


					// Horizontal - Determine Spotlight!


					// Firstly check to see how large the stage is
					var stage = wrap.find( '.__mst_stepper_list_wrapper' );

					if( stage.length > 0 ) {

						var stageWidth = Math.ceil( stage[0].getBoundingClientRect().width );

						// console.log( "Stage Width: " + stageWidth );


						var sliderWidth = par[0].getBoundingClientRect().width;

						// console.log( 'Slider Width: ' + sliderWidth );


						var upperBounds = sliderWidth - stageWidth - 10;

						// console.log( 'Upper Bounds: ' + upperBounds);


						var centerStage = stageWidth / 2;

						// console.log('Center Stage: ' + centerStage);


						var elLeftOffset = el.position().left;

						// console.log('Element Offset Left: ' + elLeftOffset);


						var calculatedOffset = ( elLeftOffset + ( el.outerWidth() / 2 ) ) - centerStage;
						// console.log('New Calculated Offset: ' + calculatedOffset);

						var movePosition;
						if( calculatedOffset <= 0 ) {
							movePosition = 0;
						} else if (calculatedOffset >= upperBounds) {
							movePosition = upperBounds;
						} else {

							movePosition = calculatedOffset;

						}

						// console.log('Move Position: ' + movePosition);


						par.css('left', '-' + movePosition + 'px');

					}



				}


			},
			calculateRelativeOffset: function( parent , child ) {

				var parentPosition = $( parent ).offset();
				var childPosition = $( child ).offset();

				var childOffset = {
					top: childPosition.top - parentPosition.top,
					left: childPosition.left - parentPosition.left
				};

				return childOffset;
			},
			calculateListHeight: function( selector ) {


				/** ;; ONLY RUN THIS STUB IF 'VERTICAL' ;;; **/
				if( selector.hasClass('__mst_stepper_vertical') ) {
					

					// if( settings.originalDirection == 'horizontal' ) {
						// console.log('Triggered!');
					// }


					// Now that the wrapper class has been determined;
					// search for the vertical wrap within
					var menuList = selector.find('.__mst_stepper_list');
					var menuListItems = menuList.find('.__mst_stepper_list_item');



					// Loop through and calculate each of the offsets for the stepper counters
					menuListItems.each(function(i, o) {

						var item = $(this);

						// Search for the item first, if it doesn't exist then append one into the 
						// parent container
						if ( item.find('.__mst_stepper_bar').length <= 0 ) {
							item.append('<div class="__mst_stepper_bar"></div>');
						}

						var bar = item.find('.__mst_stepper_bar');
						var itemTarget = item.attr('data-target');



						if(typeof itemTarget != "undefined" && itemTarget != null && itemTarget != "") {

							// Item Target has been found
							itemTarget = $( itemTarget );
							var itemHeight = itemTarget.outerHeight();
							var barHeight = (itemHeight - 30);

							bar.css('height', barHeight + 'px');


							var targetTitle = itemTarget.find( settings.titleClass );
							var offsets = pluginMethods.calculateRelativeOffset( $( selector ).find('.__mst_stepper_content_wrapper') , targetTitle );


							if( i === 0 ) {
								item.css('padding-top', ( offsets.top - 3 ) + 'px' );
							}

						}



					});

				} else {

					pluginMethods.resetBarStyles(selector);

				}

			},
			bindOnClickEvent: function( el ) {



				if( settings.direction.toLowerCase() == 'horizontal' ) {
					el.off('click', pluginMethods.listItemOnClick)
						.on('click', '.__mst_stepper_list_item', { selector: el }, pluginMethods.listItemOnClick);
				} else if ( settings.direction.toLowerCase() == 'vertical' ) {						
					el.off('click', pluginMethods.listItemOnClick)
						.on('click', settings.titleClass + ', .__mst_stepper_list_item', { selector: el }, pluginMethods.listItemOnClick);
				}



			},
			returnTitleClass: function() {

				if( settings.titleClass[0] === "." ) {
					var nTitle = settings.titleClass.substring(1, settings.titleClass.length);
					return nTitle;
				} else {
					return settings.titleClass;
				}

			},
			resetContainer: function( el ) {

				// Find all of the menu items within and remove bars
				var menuList = el.find('.__mst_stepper_list');
				var menuListItems = menuList.find('.__mst_stepper_list_item');


				// console.log(menuListItems);

				menuListItems.each(function(i, o) {


					var inEl = $(this);

					if( inEl.attr('style') ) {
						inEl.removeAttr('style');
					}

					var bars = inEl.find('.__mst_stepper_bar');
					var elTarget = inEl.attr('data-target');

					if( bars.length > 0 ) {

						// Bars have been found, lets remove them all from the DOM Element
						// bars.remove();

					}


					if( typeof elTarget != "undefined" && elTarget != null && elTarget != "" ) {


						// X-Mutate
						elTarget = $( elTarget );

						if( inEl.hasClass('current') ) {
							// Show all of the current target via 'active' class
							elTarget.addClass('active');
							inEl.addClass('active');

						} else {

							elTarget.removeClass('active');
							inEl.removeClass('active');

						}

					}


				});

			},
			resetBarStyles: function ( el ) {

				var bars = el.find('.__mst_stepper_bar');


				// console.log(bars);

				$.each(bars, function( i , o ) {


					$(this).css('height', '')
						.css('margin', '')
						.css('width', '')
						.css('background', '')
						.css('display', '');


					$(this).removeAttr('style');
				});

			},
			removeStageStyles: function( el ) {

				var list = el.find('ul.__mst_stepper_list');
				list.css('min-width', '')
					.css('left', '');


			},
			triggerResponsiveHorizontal: function( el ) {
				var current = el.find('li.__mst_stepper_list_item.current');

				current.trigger('click');
			},
			hideAllInactive: function( el ) {

				// Find the current active list item
				var list = el.find( '.__mst_stepper_list' );

				// Find al list items and remove their classes
				var listItems = list.find( '.__mst_stepper_list_item' );
				var activeItem = list.find('.__mst_stepper_list_item.current');

				// console.log( activeItem );


				$.each(listItems, function(i,o) {

					var item = $(this);


					var itemTarget = $( item.attr('data-target') );

					if( itemTarget.length > 0 ) {

						// Find the children
						var itemTargetChildren = itemTarget.children().not( settings.titleClass );

						// console.log( itemTargetChildren );

						// Item target has been found, now remove active class
						itemTargetChildren.removeClass('active').addClass('hidden');

					}


				});



				// Now set the active item to be shown
				var activeTarget = activeItem.attr('data-target');

				if( activeTarget.length > 0 ) {
					var activeChildren = $(activeTarget).children().not( settings.titleClass );
					activeChildren.addClass('active').removeClass('hidden');
				}






			}
		};

			


		pluginMethods.loopInit( selectedObjects );

		var rtimer;
		$(window).on('resize', function(ev) {
			clearTimeout( rtimer );
			rtimer = setTimeout(function() {
    			pluginMethods.loopInit( selectedObjects );
			}, 500);
		});
		

	};





}(jQuery));